package org.wildfly.swarm.config.infinispan.cache_container;

import org.wildfly.swarm.config.runtime.AttributeDocumentation;
import org.wildfly.swarm.config.runtime.ResourceDocumentation;
import org.wildfly.swarm.config.runtime.SingletonResource;
import org.wildfly.swarm.config.runtime.Addresses;
import org.wildfly.swarm.config.runtime.ResourceType;
import org.wildfly.swarm.config.runtime.Implicit;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import org.wildfly.swarm.config.runtime.ModelNodeBinding;

/**
 * Configures a cache store as write-through.
 */
@Addresses({
		"/subsystem=infinispan/cache-container=*/invalidation-cache=*/store=custom/write=through",
		"/subsystem=infinispan/cache-container=*/invalidation-cache=*/store=hotrod/write=through",
		"/subsystem=infinispan/cache-container=*/invalidation-cache=*/store=binary-jdbc/write=through",
		"/subsystem=infinispan/cache-container=*/invalidation-cache=*/store=file/write=through",
		"/subsystem=infinispan/cache-container=*/invalidation-cache=*/store=jdbc/write=through",
		"/subsystem=infinispan/cache-container=*/invalidation-cache=*/store=mixed-jdbc/write=through",
		"/subsystem=infinispan/cache-container=*/invalidation-cache=*/store=remote/write=through",
		"/subsystem=infinispan/cache-container=*/local-cache=*/store=custom/write=through",
		"/subsystem=infinispan/cache-container=*/local-cache=*/store=hotrod/write=through",
		"/subsystem=infinispan/cache-container=*/local-cache=*/store=binary-jdbc/write=through",
		"/subsystem=infinispan/cache-container=*/local-cache=*/store=file/write=through",
		"/subsystem=infinispan/cache-container=*/local-cache=*/store=jdbc/write=through",
		"/subsystem=infinispan/cache-container=*/local-cache=*/store=mixed-jdbc/write=through",
		"/subsystem=infinispan/cache-container=*/local-cache=*/store=remote/write=through",
		"/subsystem=infinispan/cache-container=*/replicated-cache=*/store=custom/write=through",
		"/subsystem=infinispan/cache-container=*/replicated-cache=*/store=hotrod/write=through",
		"/subsystem=infinispan/cache-container=*/replicated-cache=*/store=binary-jdbc/write=through",
		"/subsystem=infinispan/cache-container=*/replicated-cache=*/store=file/write=through",
		"/subsystem=infinispan/cache-container=*/replicated-cache=*/store=jdbc/write=through",
		"/subsystem=infinispan/cache-container=*/replicated-cache=*/store=mixed-jdbc/write=through",
		"/subsystem=infinispan/cache-container=*/replicated-cache=*/store=remote/write=through",
		"/subsystem=infinispan/cache-container=*/distributed-cache=*/store=custom/write=through",
		"/subsystem=infinispan/cache-container=*/distributed-cache=*/store=hotrod/write=through",
		"/subsystem=infinispan/cache-container=*/distributed-cache=*/store=binary-jdbc/write=through",
		"/subsystem=infinispan/cache-container=*/distributed-cache=*/store=file/write=through",
		"/subsystem=infinispan/cache-container=*/distributed-cache=*/store=jdbc/write=through",
		"/subsystem=infinispan/cache-container=*/distributed-cache=*/store=mixed-jdbc/write=through",
		"/subsystem=infinispan/cache-container=*/distributed-cache=*/store=remote/write=through",
		"/subsystem=infinispan/cache-container=*/scattered-cache=*/store=custom/write=through",
		"/subsystem=infinispan/cache-container=*/scattered-cache=*/store=hotrod/write=through",
		"/subsystem=infinispan/cache-container=*/scattered-cache=*/store=binary-jdbc/write=through",
		"/subsystem=infinispan/cache-container=*/scattered-cache=*/store=file/write=through",
		"/subsystem=infinispan/cache-container=*/scattered-cache=*/store=jdbc/write=through",
		"/subsystem=infinispan/cache-container=*/scattered-cache=*/store=mixed-jdbc/write=through",
		"/subsystem=infinispan/cache-container=*/scattered-cache=*/store=remote/write=through"})
@ResourceType("write")
@Implicit
public class ThroughWrite<T extends ThroughWrite<T>>
		implements
			org.wildfly.swarm.config.runtime.Keyed {

	private String key;
	private PropertyChangeSupport pcs;

	public ThroughWrite() {
		super();
		this.key = "through";
		this.pcs = new PropertyChangeSupport(this);
	}

	public String getKey() {
		return this.key;
	}

	/**
	 * Adds a property change listener
	 */
	public void addPropertyChangeListener(PropertyChangeListener listener) {
		if (null == this.pcs)
			this.pcs = new PropertyChangeSupport(this);
		this.pcs.addPropertyChangeListener(listener);
	}

	/**
	 * Removes a property change listener
	 */
	public void removePropertyChangeListener(
			java.beans.PropertyChangeListener listener) {
		if (this.pcs != null)
			this.pcs.removePropertyChangeListener(listener);
	}
}